#include "defs.h"

double betweennessCentrality(graph* G, double* BC) {
  int *S; 	/* stack of vertices in order of distance from s. Also, implicitly, the BFS queue */
  plist* P;  	/* predecessors of vertex v on shortest paths from s */
  double* sig; 	/* No. of shortest paths */
  int* d; 	/* Length of the shortest path between every pair */
  double* del; 	/* dependency of vertices */
  int *in_degree, *numEdges, *pSums;
  int *pListMem;	
  int* Srcs; 
  int *start, *end;
  int MAX_NUM_PHASES;
  int *psCount;
  int seed = 2387;
  double elapsed_time;
  int *myS, *myS_t;
  int myS_size;
  int i, j, k, p, count, myCount;
  int v, w, vert;
  int numV, num_traversals, n, m, phase_num;

  /* numV: no. of vertices to run BFS from = 2^K4approx */
  //numV = 1<<K4approx;
  n = G->nv;
  m = G->ne;
  numV = n;

  /* Permute vertices */
  Srcs = (int *) malloc(n*sizeof(int));
  for (i=0; i<n; i++) {
    Srcs[i] = i;
  }

  /* Start timing code from here */
  elapsed_time = get_seconds();
  MAX_NUM_PHASES = 2*sqrt(n);

  /* Initialize predecessor lists */
  /* Number of predecessors of a vertex is at most its in-degree. */
  P = (plist *) calloc(n, sizeof(plist));
  in_degree = (int *) calloc(n+1, sizeof(int));
  numEdges = (int *) malloc((n+1)*sizeof(int));
  pSums = (int *) malloc(sizeof(int));
  for (i=0; i<m; i++) {
    v = G->nbr[i];
    in_degree[v]++;
  }
  prefix_sums(in_degree, numEdges, pSums, n);
  pListMem = (int *) malloc(m*sizeof(int));
  for (i=0; i<n; i++) {
    P[i].list = pListMem + numEdges[i];
    P[i].degree = in_degree[i];
    P[i].count = 0;
  }
  free(in_degree);
  free(numEdges);
  free(pSums);
	
  /* Allocate shared memory */ 
  S   = (int *) malloc(n*sizeof(int));
  sig = (double *) malloc(n*sizeof(double));
  d   = (int *) malloc(n*sizeof(int));
  del = (double *) calloc(n, sizeof(double));
	
  start = (int *) malloc(MAX_NUM_PHASES*sizeof(int));
  end = (int *) malloc(MAX_NUM_PHASES*sizeof(int));
  psCount = (int *) malloc((1+1)*sizeof(int));

  /* local memory for each thread */  
  myS_size = (2*n)/1;
  myS = (int *) malloc(myS_size*sizeof(int));
  num_traversals = 0;
  myCount = 0;

  for (i=0; i<n; i++) {
    d[i] = -1;
  }
	
  /***********************************/
  /*** MAIN LOOP *********************/
  /***********************************/
  for (p=0; p<n; p++) {

		i = Srcs[p];
		if (G->firstnbr[i+1] - G->firstnbr[i] == 0) {
			continue;
		} else {
			num_traversals++;
		}

		if (num_traversals == numV + 1) {
			break;
		}
		
		sig[i] = 1;
		d[i] = 0;
		S[0] = i;
		start[0] = 0;
		end[0] = 1;
		
		count = 1;
		phase_num = 0;

		while (end[phase_num] - start[phase_num] > 0) {
			myCount = 0;
			for (vert = start[phase_num]; vert < end[phase_num]; vert++) {
				v = S[vert];
				
				for (j=G->firstnbr[v]; j<G->firstnbr[v+1]; j++) {
					w = G->nbr[j];
					if (v != w) {

						/* w found for the first time? */ 
						if (d[w] == -1) {
							if (myS_size == myCount) {
								/* Resize myS */
								myS_t = (int *)malloc(2*myS_size*sizeof(int));
								memcpy(myS_t, myS, myS_size*sizeof(int));
								free(myS);
								myS = myS_t;
								myS_size = 2*myS_size;
							}
							myS[myCount++] = w;
							d[w] = d[v] + 1;
							sig[w] = sig[v];
							P[w].list[P[w].count++] = v;
						} else if (d[w] == d[v] + 1) {
							sig[w] += sig[v];
							P[w].list[P[w].count++] = v;
						}
						
					}
				}
 			}
			
			/* Merge all local stacks for next iteration */
			phase_num++; 

			psCount[0+1] = myCount;

			start[phase_num] = end[phase_num-1];
			psCount[0] = start[phase_num];
			for(k=1; k<=1; k++) {
				psCount[k] = psCount[k-1] + psCount[k];
			}
			end[phase_num] = psCount[1];
			
			for (k = psCount[0]; k < psCount[0+1]; k++) {
				S[k] = myS[k-psCount[0]];
			} 
			
			count = end[phase_num];
		}
 	
		phase_num--;

		while (phase_num > 0) {
			for (j=start[phase_num]; j<end[phase_num]; j++) {
				w = S[j];
				for (k = 0; k < P[w].count; k++) {
					v = P[w].list[k];
					del[v] = del[v] + sig[v]*(1+del[w])/sig[w];
				}
				BC[w] += del[w];
			}

			phase_num--;
		}
		
		for (j=0; j<count; j++) {
			w = S[j];
			d[w] = -1;
			del[w] = 0;
			P[w].count = 0;
		}
  }
  /***********************************/
  /*** END OF MAIN LOOP **************/
  /***********************************/
 

  free(myS);
	
  free(S);
  free(pListMem);
  free(P);
  free(sig);
  free(d);
  free(del);
  free(start);
  free(end);
  free(psCount);
  elapsed_time = get_seconds() - elapsed_time;
  free(Srcs);

  return elapsed_time;
}

